"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.mapSchema = void 0;
const index_js_1 = require("../feature-libraries/index.js");
const proxies_js_1 = require("./proxies.js");
const proxyBinding_js_1 = require("./proxyBinding.js");
const index_js_2 = require("./core/index.js");
const schemaTypes_js_1 = require("./schemaTypes.js");
const index_js_3 = require("./core/index.js");
const toMapTree_js_1 = require("./toMapTree.js");
const toFlexSchema_js_1 = require("./toFlexSchema.js");
const index_js_4 = require("../util/index.js");
const treeNodeValid_js_1 = require("./treeNodeValid.js");
const handler = {
    getPrototypeOf: () => {
        return Map.prototype;
    },
};
class CustomMapNodeBase extends treeNodeValid_js_1.TreeNodeValid {
    [Symbol.iterator]() {
        return this.entries();
    }
    delete(key) {
        const field = (0, proxyBinding_js_1.getOrCreateInnerNode)(this).getBoxed(key);
        field.editor.set(undefined, field.length === 0);
    }
    *entries() {
        const node = (0, proxyBinding_js_1.getOrCreateInnerNode)(this);
        for (const key of node.keys()) {
            yield [
                key,
                (0, proxies_js_1.getTreeNodeForField)(node.getBoxed(key)),
            ];
        }
    }
    get(key) {
        const node = (0, proxyBinding_js_1.getOrCreateInnerNode)(this);
        const field = node.getBoxed(key);
        return (0, proxies_js_1.getTreeNodeForField)(field);
    }
    has(key) {
        return (0, proxyBinding_js_1.getOrCreateInnerNode)(this).tryGetField((0, index_js_4.brand)(key)) !== undefined;
    }
    keys() {
        const node = (0, proxyBinding_js_1.getOrCreateInnerNode)(this);
        return node.keys();
    }
    set(key, value) {
        const node = (0, proxyBinding_js_1.getOrCreateInnerNode)(this);
        const classSchema = (0, index_js_2.getSimpleNodeSchema)(node.schema);
        const mapTree = (0, toMapTree_js_1.mapTreeFromNodeData)(value, (0, schemaTypes_js_1.createFieldSchema)(schemaTypes_js_1.FieldKind.Optional, classSchema.info), node.context?.nodeKeyManager, (0, index_js_1.getSchemaAndPolicy)(node));
        const field = node.getBoxed(key);
        if (node.context !== undefined) {
            (0, proxies_js_1.prepareContentForHydration)(mapTree, node.context.checkout.forest);
        }
        field.editor.set(mapTree, field.length === 0);
        return this;
    }
    get size() {
        return (0, index_js_4.count)((0, proxyBinding_js_1.getOrCreateInnerNode)(this).keys());
    }
    *values() {
        for (const [, value] of this.entries()) {
            yield value;
        }
    }
    forEach(callbackFn, thisArg) {
        for (const field of (0, proxyBinding_js_1.getOrCreateInnerNode)(this).boxedIterator()) {
            const node = (0, proxies_js_1.getTreeNodeForField)(field);
            callbackFn.call(thisArg, node, field.key, this);
        }
    }
}
CustomMapNodeBase.kind = index_js_3.NodeKind.Map;
/**
 * Define a {@link TreeNodeSchema} for a {@link (TreeArrayNode:interface)}.
 *
 * @param base - base schema type to extend.
 * @param useMapPrototype - should this type emulate a ES6 Map object (by faking its prototype with a proxy).
 */
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
function mapSchema(identifier, info, implicitlyConstructable, useMapPrototype) {
    let flexSchema;
    class schema extends CustomMapNodeBase {
        static prepareInstance(instance, flexNode) {
            if (useMapPrototype) {
                return new Proxy(instance, handler);
            }
            return instance;
        }
        static buildRawNode(instance, input) {
            return (0, index_js_1.getOrCreateMapTreeNode)(flexSchema, (0, toMapTree_js_1.mapTreeFromNodeData)(input, this));
        }
        static oneTimeSetup() {
            flexSchema = (0, toFlexSchema_js_1.getFlexSchema)(this);
        }
        // eslint-disable-next-line import/no-deprecated
        get [index_js_3.typeNameSymbol]() {
            return identifier;
        }
        get [index_js_3.typeSchemaSymbol]() {
            return schema.constructorCached?.constructor;
        }
    }
    schema.constructorCached = undefined;
    schema.identifier = identifier;
    schema.info = info;
    schema.implicitlyConstructable = implicitlyConstructable;
    const schemaErased = schema;
    return schemaErased;
}
exports.mapSchema = mapSchema;
//# sourceMappingURL=mapNode.js.map